package models

import (
	"gogs.mikescher.com/BlackForestBytes/goext/cryptext"
	"gogs.mikescher.com/BlackForestBytes/goext/langext"
	"gogs.mikescher.com/BlackForestBytes/goext/rfctime"
)

type AppUserAuthType string //@enum:type

const (
	AuthTypeMailPassword        AppUserAuthType = "MAIL_PASSWORD"
	AuthTypePhoneNumberFirebase AppUserAuthType = "FIREBASE_PHONE"
	AuthTypeAppleSignIn         AppUserAuthType = "APPLE_SIGNIN"
	AuthTypeGoogleSignIn        AppUserAuthType = "GOOGLE_SIGNIN"
)

// AppUser is a normal Baden-News User
type AppUser struct {
	UserID       AppUserID                `json:"id"           bson:"_id,omitempty"`
	AuthType     AppUserAuthType          `json:"authType"     bson:"authType"`
	Email        *string                  `json:"email"        bson:"email"`
	Password     cryptext.PassHash        `json:"-"            bson:"password"`
	CreationTime rfctime.RFC3339NanoTime  `json:"creationTime" bson:"creationTime"`
	Deleted      *rfctime.RFC3339NanoTime `json:"deleted"      bson:"deleted"`
}

func (u AppUser) GetFirstName() string {
	return "" //TODO AppUser firstname
}

func (u AppUser) GetLastName() string {
	return "" //TODO AppUser lastname
}

func (u AppUser) GetUserID() AnyID {
	return u.UserID.AsAny()
}

func (u AppUser) GetEmail() string {
	return langext.Coalesce(u.Email, "")
}

func (u AppUser) GetUsername() string {
	return u.UserID.String() //TODO AppUser username
}

func (u AppUser) Type() UserType {
	return UserTypApplication
}
